/*-
 * LstForeach.c --
 *	Perform a given function on all elements of a list starting from
 *	a given point.
 *
 * Copyright (c) 1988 by University of California Regents
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appears in all copies.  Neither the University of California nor
 * Adam de Boor makes any representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */
#ifndef lint
static char *rcsid =
"$Id: lstForEachFrom.c,v 1.4 88/11/17 20:52:50 adam Exp $ SPRITE (Berkeley)";
#endif lint

#include	"lstInt.h"

/*-
 *-----------------------------------------------------------------------
 * Lst_ForEachFrom --
 *	Apply the given function to each element of the given list. The
 *	function should return 0 if Lst_ForEach should continue and non-
 *	zero if it should abort.
 *
 * Results:
 *	None.
 *
 * Side Effects:
 *	Only those created by the passed-in function.
 *
 *-----------------------------------------------------------------------
 */
/*VARARGS2*/
void
Lst_ForEachFrom (ln, proc, d)
    LstNode    	  	ln;
    register int	(*proc)();
    register ClientData	d;
{
    register ListNode	tln = (ListNode)ln;
    register List 	list = (List)tln->lPtr;
    register ListNode	next;
    
    if (!LstValid (list) || LstIsEmpty (list)) {
	return;
    }
    
    do {
	/*
	 * Take care of having the current element deleted out from under
	 * us. Still doesn't take care of having the next element deleted from
	 * under us, but that's much less likely.
	 */
	next = tln->nextPtr;
	if ((*proc) (tln->datum, d) != 0) {
	    break;
	} else {
	    tln = next;
	}
    } while (!LstIsEmpty(list) &&
	     (tln != list->firstPtr) &&
	     (tln != NilListNode));
    
}

